package org.openxma.dsl.pom.naming;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.naming.QualifiedName;
import org.eclipse.xtext.util.SimpleAttributeResolver;

import at.spardat.xma.guidesign.BDAttribute;
import at.spardat.xma.guidesign.HiddenWidget;
import at.spardat.xma.guidesign.NotebookPage;
import at.spardat.xma.guidesign.SimpleCombo;
import at.spardat.xma.guidesign.XMACombo;
import at.spardat.xma.guidesign.XMACompProperty;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAComposite;
import at.spardat.xma.guidesign.XMAList;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMATable;
import at.spardat.xma.guidesign.XMATableColumn;
import at.spardat.xma.guidesign.XMAText;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.XMAFunction;
import at.spardat.xma.guidesign.util.GuidesignSwitch;

import com.google.common.base.Function;
import com.google.inject.Inject;

public class GuiDesignNameFunction extends GuidesignSwitch implements Function<EObject, String> {
	private SimpleAttributeResolver<EObject, String> resolver = SimpleAttributeResolver.newResolver(String.class, "name");
	
	public String apply(EObject from) {
		return (String) doSwitch(from);
	}

	@Override
	public Object caseXMAComposite(XMAComposite object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseXMAText(XMAText object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseSimpleCombo(SimpleCombo object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseHiddenWidget(HiddenWidget object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseXMATable(XMATable object) {
	    return object.getNamInstance();
	}

	@Override
	public Object caseXMAList(XMAList object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseXMACombo(XMACombo object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseXMAWidget(XMAWidget object) {
		return object.getNamInstance();
	}

	@Override
	public Object caseXMATableColumn(XMATableColumn object) {
	    return object.getNamInstance();
	}

	@Override
	public Object caseXMAPage(XMAPage object) {
		return object.getNamClass();
	}

	@Override
	public Object caseNotebookPage(NotebookPage object) {
		return object.getNamClass();
	}

	@Override
	public Object caseXMACompProperty(XMACompProperty object) {
		return object.getNamProperty();
	}

	@Override
	public Object caseBDAttribute(BDAttribute object) {
		return object.getBdObject().getNamInstance() + "." + object.getNamAttrVal();
	}

	@Override
	public Object caseXMAComponent(XMAComponent object) {
		return object.getNamPackage() + "." + object.getNamClass();
	}

	@Override
	public Object defaultCase(EObject object) {
		if (object instanceof XMAFunction) {
			return ((XMAFunction) object).getNamFunction();
		}
		return resolver.apply(object);
	}
	
	public static class QualifiedNameAdapter implements Function<EObject, QualifiedName> {
	    
	    @Inject
	    GuiDesignNameFunction nameFunction;

        public QualifiedName apply(EObject from) {
            String name = nameFunction.apply(from);
            return name != null ? QualifiedName.create(name) : null;
        }
	    
	}
	
}
